==> 1.6.12&file=out_yasapi-1.6.12-src.tar.gz: out_yasapi-1.6.12/out_yasapi/yasapi_trace.c <==
/*
 * yasapi_trace.c
 * Copyright (C) 2015-2016 Peter Belkner <pbelkner@snafu.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yasapi.h>
#if defined (YASAPI_DEBUG) // {
#include <resources.h>

///////////////////////////////////////////////////////////////////////////////
static const Property gcaTraceProperties[]={
  { YASAPI_PROPERTY_DEBUG,&gcIntType,offsetof(Trace,nDebug),0 },
  { YASAPI_PROPERTY_FILE,&gcIntType,offsetof(Trace,bFile),0 },
  { YASAPI_PROPERTY_SLEEP,&gcIntType,offsetof(Trace,nSleep),0 },
  { NULL,NULL,0,0 }
};

///////////////////////////////////////////////////////////////////////////////
static const ControlComboBoxList gcaTraceDebugList[]={
  { L"Off",0 },
  { L"Default",1 },
  { L"Verbose 1",2 },
  { L"Verbose 2",3 },
  { L"Verbose 3",4 },
  { NULL,0 }
};

static const ControlComboBoxConfig gcTraceDebugConfig=
  { IDC_COMBOBOX_DEBUG,offsetof(Trace,nDebug),gcaTraceDebugList,
      L"Whether to switch off the debug console or whether to write"
      L" in default or verbose mode to it." };

////////////////
static const ControlCheckBoxConfig gcTraceFileConfig=
  { IDC_CHECKBOX_FILE,offsetof(Trace,bFile),0,1,
      L"Whether YASAPI should write the trace to a file or to a console." };

////////////////
static const ControlComboBoxList gcaTraceSleepList[]={
  { L"0 sec",0 },
  { L"2 sec",2*1000 },
  { L"5 sec",5*1000 },
  { L"10 sec",10*1000 },
  { L"20 sec",20*1000 },
  { L"1 min",1*60*1000 },
  { L"2 min",2*60*1000 },
  { L"5 min",5*60*1000 },
  { NULL,0 }
};

static const ControlComboBoxConfig gcTraceSleepConfig=
  { IDC_COMBOBOX_SLEEP,offsetof(Trace,nSleep),gcaTraceSleepList,
      L"When switched on, how long should the debug console survive closing"
      L" Winamp." };

const Control gcaTraceControls[]={
  { &gcComboBoxType,&gcTraceDebugConfig },
  { &gcCheckBoxType,&gcTraceFileConfig },
  { &gcComboBoxType,&gcTraceSleepConfig },
  { NULL,0 }
};

///////////////////////////////////////////////////////////////////////////////
Trace trace;

int TraceCreate(Trace *pTrace, const wchar_t *path)
{
  PropertyIOConfig c={
    path,                 // const wchar_t *path;
    pTrace,               // void *pData;
    TraceDefault(),       // const void *pDefault;
    YASAPI_GROUP_COMMON,  // const wchar_t *group;
    L"    "               // const wchar_t *pfx;
  };

  wchar_t filepath[MAX_PATH];
  wchar_t *pp;
  size_t len;
  SYSTEMTIME time;

  if (path) {
    pTrace->path=path;
    PropertiesLoad(gcaTraceProperties,&c);
  }

  if (!pTrace->nDebug)
    pTrace->tag=TRACE_CLOSE;
  else if (pTrace->bFile) {
    pTrace->tag=TRACE_FILE;

    wcscpy(filepath,pTrace->path);
    len=wcslen(filepath)-4;
    pp=filepath+len;

    GetSystemTime(&time);

    swprintf(pp,
      (sizeof filepath)-len,
      L"-%04d"            // wYear
      L"-%02d"            // wMonth
      L"-%02d"            // wDay
      L"-%02d"            // wHour
      L"-%02d"            // wMinute
      L"-%02d"            // wSecond
      L"-%04d"            // wMilliseconds
      L".txt",
      time.wYear,
      time.wMonth,
      time.wDay,
      time.wHour,
      time.wMinute,
      time.wSecond,
      time.wMilliseconds
    );

    pTrace->f=_wfopen(filepath,L"w");;
  }
  else {
    pTrace->tag=TRACE_CONSOLE;
    pTrace->f=stderr;
    ConsoleOpen();
  }

  return 0;
}

void TraceDestroy(Trace *pTrace, int nSleep)
{
  switch (pTrace->tag) {
  case TRACE_FILE:
    fclose(pTrace->f);
    break;
  case TRACE_CONSOLE:
    if (0<nSleep) {
      fprintf(stderr,"  sleeping %d ms ...\n",nSleep);
      Sleep(nSleep);
    }

    ConsoleClose();
    break;
  default:
    break;
  }

  pTrace->tag=TRACE_CLOSE;
}

int TraceSwitch(Trace *pTrace)
{
  PropertyIOConfig c={
    pTrace->path,         // const wchar_t *path;
    pTrace,               // void *pData;
    NULL,                 // const void *pDefault;
    YASAPI_GROUP_COMMON,  // const wchar_t *group;
    L"    "               // const wchar_t *pfx;
  };

  int tag;

  PropertiesSave(gcaTraceProperties,&c);
  tag=!pTrace->nDebug?TRACE_CLOSE:pTrace->bFile?TRACE_FILE:TRACE_CONSOLE;

  if (tag!=pTrace->tag) {
    TraceDestroy(pTrace,0);

    return TraceCreate(pTrace,NULL);
  }
  else
    return 0;
}

int tputs(Trace *pTrace, int nLevel, const char *s)
{
  if (TRACE_CLOSE<pTrace->tag&&nLevel<pTrace->nDebug) {
    fputs(s,pTrace->f);
    fflush(pTrace->f);
  }

  return 0;
}

int tputws(Trace *pTrace, int nLevel, const wchar_t *ws)
{
  if (TRACE_CLOSE<pTrace->tag&&nLevel<pTrace->nDebug) {
    fputws(ws,pTrace->f);
    fflush(pTrace->f);
  }

  return 0;
}

int tvprintf(Trace *pTrace, const char *format, va_list ap)
{
  if (TRACE_CLOSE<pTrace->tag) {
    vfprintf(pTrace->f,format,ap);
    fflush(pTrace->f);
  }

  return 0;
}

int tprintf(Trace *pTrace, int nLevel, const char *format, ...)
{
  va_list ap;

  if (TRACE_CLOSE<pTrace->tag&&nLevel<pTrace->nDebug) {
    va_start(ap,format);
    tvprintf(pTrace,format,ap);
    va_end(ap);
  }

  return 0;
}

int twprintf(Trace *pTrace, int nLevel, const wchar_t *format, ...)
{
  va_list ap;

  if (TRACE_CLOSE<pTrace->tag&&nLevel<pTrace->nDebug) {
    va_start(ap,format);
    vfwprintf(pTrace->f,format,ap);
    va_end(ap);
    fflush(pTrace->f);
  }

  return 0;
}

Trace *TraceDefault()
{
  static int bInitialized;
  static Trace trace;

  if (!bInitialized) {
    trace.nDebug=0;
    trace.bFile=0;
    trace.nSleep=5000;
    bInitialized=1;
  }

  return &trace;
}
#endif // }

==> 1.7.3&file=out_yasapi-1.7.3-src.tar.gz: out_yasapi-1.7.3/out_yasapi/yasapi_trace.c <==
/*
 * yasapi_trace.c
 * Copyright (C) 2015-2016 Peter Belkner <pbelkner@snafu.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yasapi.h>
#if defined (YASAPI_DEBUG) // {
#include <resources.h>

///////////////////////////////////////////////////////////////////////////////
static const Property gcaTraceProperties[]={
  { YASAPI_PROPERTY_DEBUG,&gcIntType,offsetof(Trace,nDebug),0 },
  { YASAPI_PROPERTY_FILE,&gcIntType,offsetof(Trace,bFile),0 },
  { YASAPI_PROPERTY_SLEEP,&gcIntType,offsetof(Trace,nSleep),0 },
  { NULL,NULL,0,0 }
};

///////////////////////////////////////////////////////////////////////////////
static const ControlComboBoxList gcaTraceDebugList[]={
  { L"Off",0 },
  { L"Default",1 },
  { L"Verbose 1",2 },
  { L"Verbose 2",3 },
  { L"Verbose 3",4 },
  { NULL,0 }
};

static const ControlComboBoxConfig gcTraceDebugConfig=
  { IDC_COMBOBOX_DEBUG,offsetof(Trace,nDebug),gcaTraceDebugList,
      L"Whether to switch off the debug console or whether to write"
      L" in default or verbose mode to it." };

////////////////
static const ControlCheckBoxConfig gcTraceFileConfig=
  { IDC_CHECKBOX_FILE,offsetof(Trace,bFile),0,1,
      L"Whether YASAPI should write the trace to a file or to a console." };

////////////////
static const ControlComboBoxList gcaTraceSleepList[]={
  { L"0 sec",0 },
  { L"2 sec",2*1000 },
  { L"5 sec",5*1000 },
  { L"10 sec",10*1000 },
  { L"20 sec",20*1000 },
  { L"1 min",1*60*1000 },
  { L"2 min",2*60*1000 },
  { L"5 min",5*60*1000 },
  { NULL,0 }
};

static const ControlComboBoxConfig gcTraceSleepConfig=
  { IDC_COMBOBOX_SLEEP,offsetof(Trace,nSleep),gcaTraceSleepList,
      L"When switched on, how long should the debug console survive closing"
      L" Winamp." };

const Control gcaTraceControls[]={
  { &gcComboBoxType,&gcTraceDebugConfig },
  { &gcCheckBoxType,&gcTraceFileConfig },
  { &gcComboBoxType,&gcTraceSleepConfig },
  { NULL,0 }
};

///////////////////////////////////////////////////////////////////////////////
Trace trace;

int TraceCreate(Trace *pTrace, const char *label, const wchar_t *path)
{
  PropertyIOConfig c={
    path,                 // const wchar_t *path;
    pTrace,               // void *pData;
    TraceDefault(),       // const void *pDefault;
    YASAPI_GROUP_COMMON,  // const wchar_t *group;
    L"    "               // const wchar_t *pfx;
  };

  wchar_t filepath[MAX_PATH];
  wchar_t *pp;
  size_t len;
  SYSTEMTIME time;

  if (label)
    pTrace->label=label;

  if (path) {
    pTrace->path=path;
    PropertiesLoad(gcaTraceProperties,&c);
  }

  if (!pTrace->nDebug) {
    pTrace->tag=TRACE_CLOSE;
    pTrace->f=NULL;
  }
  else if (pTrace->bFile) {
    pTrace->tag=TRACE_FILE;

    wcscpy(filepath,pTrace->path);
    len=wcslen(filepath)-4;
    pp=filepath+len;

    GetSystemTime(&time);

    swprintf(pp,
      (sizeof filepath)-len,
      L"-%04d"            // wYear
      L"-%02d"            // wMonth
      L"-%02d"            // wDay
      L"-%02d"            // wHour
      L"-%02d"            // wMinute
      L"-%02d"            // wSecond
      L"-%04d"            // wMilliseconds
      L".txt",
      time.wYear,
      time.wMonth,
      time.wDay,
      time.wHour,
      time.wMinute,
      time.wSecond,
      time.wMilliseconds
    );

    pTrace->f=_wfopen(filepath,L"w");;
  }
  else {
    pTrace->tag=TRACE_CONSOLE;
    pTrace->f=stderr;
    ConsoleOpen();
  }

  if (NULL!=pTrace->f&&NULL!=pTrace->label) {
    fprintf(pTrace->f,"%s\n",pTrace->label);
    fflush(pTrace->f);
  }

  return 0;
}

void TraceDestroy(Trace *pTrace, int nSleep)
{
  switch (pTrace->tag) {
  case TRACE_FILE:
    fclose(pTrace->f);
    break;
  case TRACE_CONSOLE:
    if (0<nSleep) {
      fprintf(stderr,"  sleeping %d ms ...\n",nSleep);
      Sleep(nSleep);
    }

    ConsoleClose();
    break;
  default:
    break;
  }

  pTrace->tag=TRACE_CLOSE;
}

int TraceSwitch(Trace *pTrace)
{
  PropertyIOConfig c={
    pTrace->path,         // const wchar_t *path;
    pTrace,               // void *pData;
    NULL,                 // const void *pDefault;
    YASAPI_GROUP_COMMON,  // const wchar_t *group;
    L"    "               // const wchar_t *pfx;
  };

  int tag;

  PropertiesSave(gcaTraceProperties,&c);
  tag=!pTrace->nDebug?TRACE_CLOSE:pTrace->bFile?TRACE_FILE:TRACE_CONSOLE;

  if (tag!=pTrace->tag) {
    TraceDestroy(pTrace,0);

    return TraceCreate(pTrace,NULL,NULL);
  }
  else
    return 0;
}

int tputs(Trace *pTrace, int nLevel, const char *s)
{
  if (TRACE_CLOSE<pTrace->tag&&nLevel<pTrace->nDebug) {
    fputs(s,pTrace->f);
    fflush(pTrace->f);
  }

  return 0;
}

int tputws(Trace *pTrace, int nLevel, const wchar_t *ws)
{
  if (TRACE_CLOSE<pTrace->tag&&nLevel<pTrace->nDebug) {
    fputws(ws,pTrace->f);
    fflush(pTrace->f);
  }

  return 0;
}

int tvprintf(Trace *pTrace, const char *format, va_list ap)
{
  if (TRACE_CLOSE<pTrace->tag) {
    vfprintf(pTrace->f,format,ap);
    fflush(pTrace->f);
  }

  return 0;
}

int tprintf(Trace *pTrace, int nLevel, const char *format, ...)
{
  va_list ap;

  if (TRACE_CLOSE<pTrace->tag&&nLevel<pTrace->nDebug) {
    va_start(ap,format);
    tvprintf(pTrace,format,ap);
    va_end(ap);
  }

  return 0;
}

int twprintf(Trace *pTrace, int nLevel, const wchar_t *format, ...)
{
  va_list ap;

  if (TRACE_CLOSE<pTrace->tag&&nLevel<pTrace->nDebug) {
    va_start(ap,format);
    vfwprintf(pTrace->f,format,ap);
    va_end(ap);
    fflush(pTrace->f);
  }

  return 0;
}

Trace *TraceDefault()
{
  static int bInitialized;
  static Trace trace;

  if (!bInitialized) {
    trace.nDebug=0;
    trace.bFile=0;
    trace.nSleep=5000;
    bInitialized=1;
  }

  return &trace;
}
#endif // }
